# -*- encoding=utf-8 -*-
import sys,os,time
from subprocess import Popen, PIPE
from math import *

def generate_benchmark(base_name, sphereRadius, nbSpheres, nbIter=10000):
	O.reset()

	Density=2400
	frictionAngle=radians(35)
	tc = 0.001 
	en = 0.3   
	es = 0.3

	params=utils.getViscoelasticFromSpheresInteraction(10e3,tc,en,es)
	imported = utils.import_stl_geometry('baraban.stl',frictionAngle=frictionAngle,physParamsClass="SimpleViscoelasticBodyParameters",**params) 

	for i in xrange(nbSpheres[0]):
		for j in xrange(nbSpheres[1]):
			for k in xrange(nbSpheres[2]):
				x = (i*2 - nbSpheres[0]+1)*sphereRadius*1.1
				y = (j*2 - nbSpheres[1]+1)*sphereRadius*1.1
				z = (k*2 - nbSpheres[2]+2)*sphereRadius*1.1
				color = (0,0,1)
				if i>nbSpheres[0]/2-1: color = (1,0,0)
				s=utils.sphere([x,y,z],sphereRadius,color=color,density=Density,frictionAngle=frictionAngle,physParamsClass="SimpleViscoelasticBodyParameters")
				p=utils.getViscoelasticFromSpheresInteraction(s.phys['mass'],tc,en,es)
				s.phys['kn'],s.phys['cn'],s.phys['ks'],s.phys['cs']=p['kn'],p['cn'],p['ks'],p['cs']
				O.bodies.append(s)

	O.initializers=[
		BoundingVolumeMetaEngine([InteractingSphere2AABB(),InteractingFacet2AABB(),MetaInteractingGeometry2AABB()])
		]

	O.engines=[
		BexResetter(),
		BoundingVolumeMetaEngine([
			InteractingSphere2AABB(),
			InteractingFacet2AABB(),
			MetaInteractingGeometry2AABB()
		]),
		InsertionSortCollider(),
		InteractionGeometryMetaEngine([
			InteractingSphere2InteractingSphere4SpheresContactGeometry(),
			InteractingFacet2InteractingSphere4SpheresContactGeometry()
		]),
		InteractionPhysicsMetaEngine([SimpleViscoelasticRelationships()]),
		ConstitutiveLawDispatcher([ef2_Spheres_Viscoelastic_SimpleViscoelasticContactLaw()]),
		GravityEngine(gravity=[0,-9.81,0]),
		NewtonsDampedLaw(damping=0),
		RotationEngine(subscribedBodies=imported,rotationAxis=[0,0,1],rotateAroundZero=True,angularVelocity=1.047),
		SQLiteRecorder(recorders=['se3'],dbFile=base_name+'-gen.sqlite',virtPeriod=0.04)
	]

	O.dt=.0002 #.2*tc

	print '\nGENERATE BENCHMARK %s:' % base_name
	shapes = {}
	print 'Bodies:',
	for b in O.bodies:
		sh = b.shape.name
		if not shapes.has_key(sh): shapes[sh] = 0
		shapes[sh] +=1
	for k in shapes:
		print '%s: %d'%(k,shapes[k]),
	print ''

	print 'Run: %d iters...'%nbIter,
	sys.stdout.flush()
	time_start = time.time()
	O.run(nbIter,True)
	time_end = time.time()
	print 'elapsed: %.2f sec' % (time_end-time_start)

	file_name = base_name+'.xml.bz2'
	print 'Save scene to %s' % file_name
	O.save(file_name)

	print "Done."

def run_benchmark(base_name, nbIter, nbRun):
	O.reset()

	print '\nRUN BENCHMARK %s:' % base_name
	scene_file_name = base_name+'.xml.bz2'
	print 'loading scene %s...'%scene_file_name,
	sys.stdout.flush()
	start_time = time.time()
	O.load(scene_file_name)
	stop_time = time.time()
	print 'elapsed: %.2f sec' % (stop_time-start_time)

	shapes = {}
	print 'Bodies:',
	for b in O.bodies:
		sh = b.shape.name
		if not shapes.has_key(sh): shapes[sh] = 0
		shapes[sh] +=1
	for k in shapes:
		print '%s: %d'%(k,shapes[k]),
	print ''

	num_threads_key='OMP_NUM_THREADS'
	num_threads=1
	if os.environ.has_key(num_threads_key):
		num_threads=int(os.environ[num_threads_key])
	print '* '+num_threads_key+': %d'%num_threads

	perf=[]
	elaps=[]
	utils.typedEngine('SQLiteRecorder')['dbFile']=base_name+'-run.sqlite'
	utils.typedEngine('SQLiteRecorder')['nDone']=0
	for i in xrange(1,nbRun+1):
		print 'Run %d(%d): %d iters...'%(i,nbRun,nbIter),
		sys.stdout.flush()
		time_start=time.time()
		O.run(nbIter,True)
		time_end=time.time()
		elaps += [ time_end-time_start ]
		perf += [ nbIter/elaps[-1] ]
		print 'elapsed: %.2f sec, performance: %.2f iter/sec' % (elaps[-1], perf[-1]) 
	print 'Done. Elapsed: %.2f sec, perfomance: %.2f iter/sec' % (sum(elaps),sum(perf)/len(perf))

	output_file = num_threads_key+'=%d.dat'%num_threads
	if not os.path.exists(output_file):
		with open(output_file,'w') as of:
			proc_cmd='cat /proc/cpuinfo | grep model\ name | uniq | sed \'s/.\+:\ //\''
			p = Popen(proc_cmd, shell=True, stdout=PIPE)
			proc, errors = p.communicate()
			print >> of, '# Processor: '+proc[:-1]
			print >> of, '# Benchmark\tSpheres\tIters\tElapsed, sec\tPerfomance, sec/iter' 
	with open(output_file,'a') as of:
		print >> of, '%s\t%d\t%d\t%.2f\t%.2f'%(base_name,shapes['Sphere'],nbIter*nbRun,sum(elaps), sum(elaps)/nbIter/nbRun)


# Generating

generate_benchmark(base_name='scene1k',sphereRadius=0.25, nbSpheres=(10,10,10), nbIter=10000)
generate_benchmark(base_name='scene5k',sphereRadius=0.2, nbSpheres=(18,12,20), nbIter=10000)
generate_benchmark(base_name='scene37k',sphereRadius=0.1, nbSpheres=(37,25,40), nbIter=10000)
generate_benchmark(base_name='scene100k',sphereRadius=0.07, nbSpheres=(51,35,56), nbIter=10000)

# Running

run_benchmark(base_name='scene1k', nbIter=10000, nbRun=3)
run_benchmark(base_name='scene5k', nbIter=10000, nbRun=3)
run_benchmark(base_name='scene37k', nbIter=10000, nbRun=3)
run_benchmark(base_name='scene100k', nbIter=10000, nbRun=3)

sys.exit(0)
